//-----------------------------------------------------------------------------
//
// BLUE: A ROOT class implementing the Best Linear Unbiased Estimate method.
// 
// Copyright (C) 2012-2013, Richard.Nisius@mpp.mpg.de
// All rights reserved
//
// This file is part of BLUE - Version 1.8.0.
//
// BLUE is free software: you can redistribute it and/or modify it under the 
// terms of the GNU Lesser General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// For the licensing terms see the file COPYING or http://www.gnu.org/licenses.
//
//-----------------------------------------------------------------------------
#include "Riostream.h"
#include "TMath.h"
#include "Blue.h"

void B_Peelles(){

  //---------------------------------------------------------------------------------
  // Only one result is calculated
  //---------------------------------------------------------------------------------

  // The number of estimates, uncertainties and observables
  static const Int_t NumEst = 2;
  static const Int_t NumUnc = 2;

  // Fill estimates
  static const Int_t LenXEst = NumEst * (NumUnc+1);
  Double_t XEst[LenXEst] = {
    8.0, 0.16, 0.80,
    8.5, 0.17, 0.85
  };

  // Construct Object
  Blue *myBlue = new Blue(NumEst, NumUnc);
  myBlue->PrintStatus();
  
  // Fill all estimates
  Int_t ind = 0;
  for(Int_t i = 0; i<NumEst; i++){
    myBlue->FillEst(i,&XEst[ind]);
    ind = ind + NumUnc + 1;
  }

  // Fill correlation
  myBlue->FillCor(0,0.0);
  myBlue->FillCor(1,1.0);
  
  // Fix input and inspect input
  myBlue->FixInp();
  myBlue->PrintEst();
  myBlue->PrintCor(0);
  myBlue->PrintCov();
  myBlue->PrintRho();

  // Inspect pair
  myBlue->InspectPair(0,1,"B_Peelles");

  // Solve 
  myBlue->Solve();
  printf("... B_Peelles: The result \n");
  myBlue->PrintResult();

  // Delete Object
  delete myBlue;
}
